<?php

// Exit if this file is accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;


/**
 * An instance of this class represents the Simple Shipping Labels plugin.
 * This plugin adds a column of buttons in WooCommerce orders page,
 * to generate a single or bulk of editable shipping labels.
 */
class dimap_SimpleShippingLabels {

	  //////////////////////////////////////////////////
	 /// Default label settings and class constants ///
	//////////////////////////////////////////////////
	// Plugin and settings slugs/names/ids/... used in the functions below.
	const PLUGIN_VERSION = '1.0.0';
	const PLUGIN_NAME = 'Simple Shipping Labels';
	const OPTION_GROUP_NAME = 'sslabels_plugin_options';
	const OPTION_NAME = 'simple_shipping_labels';
	const SETTINGS_PAGE_SLUG = 'sslabels_settings';
	const SETTINGS_PAGE_SECTION_SLUG = 'general';

	/*** Plugin settings ***/
	public $settings = array();
	public $default_settings = array(
		'label_height'				=>	53,		// mm
		'label_width'				=>	101,	// mm
		'label_padding_vertical'	=>	2,		// mm
		'label_padding_horizontal'	=>	2,		// mm
		'test_labels'				=>	'',
		'hide_base_country'			=>	'on',	// checked checkbox
		'recipient_align_fields'	=>	'center',
		'plugin_version'			=> 	self::PLUGIN_VERSION,
	);

	public $recipient_align_fields_options = ['center','left','right'];

	  ////////////////////////////////////////////////
	 /// Class constructor, methods and callbacks ///
	////////////////////////////////////////////////
	/**
	 * The plugin class constructor is responsible for:
	 * - Getting saved plugin options from wp_options database table or using the $default_settings array above.
	 * - Registering plugin hooks:
	 *   - Run plugin update routine when plugin loads.
	 *   - Add settings page to the admin menu.
	 *   - Add buttons and JavaScript to WooCommerce orders page to generate labels.
	 *   - Register custom AJAX endpoint (a single function acts as plugin api) to generate labels page or reset plugin options.
	 */
	public function __construct()
    {
		// ###########################
		// ### Set labels settings ###
		// ###########################
		// Use stored plugin settings if a plugin option exists in database.
		$option = get_option( self::OPTION_NAME );
		if ($option) {
			$this->settings = $option;
		} else {
			$this->settings = $this->default_settings;
		}
		

		// #######################
		// ### WordPress hooks ###
		// #######################
		// Add plugin version check and update routine.
		// This hook runs each time the plugin is loaded (wordpress environment loads),
		// because there are no other reliable hooks that run when plugin files are
		// updated from remote server.
		add_action('plugins_loaded', array( $this, 'check_plugin_version' ) );

		// Add plugin settings page ("options page") and register settings.
		add_action( 'admin_menu', array( $this, 'add_plugin_options_page' ) );
		add_action( 'admin_init', array( $this, 'register_settings' ) );

		// Register AJAX endpoint as plugin API to generate labels or reset settings to default.
		add_action( 'wp_ajax_sslabels_api', array( $this, 'sslabels_api' ) );


		// #########################
		// ### WooCommerce hooks ###
		// #########################
		// Add custom column to WooCommerce orders table page.
		add_filter( 'manage_edit-shop_order_columns', array( $this, 'add_print_buttons_column' ) );
		
		// Populate the column with print buttons.
		add_action( 'manage_shop_order_posts_custom_column', array( $this, 'add_print_buttons' ) );

		// Add JavaScript functionality to the buttons.
		add_action( 'admin_enqueue_scripts', array( $this, 'add_orders_page_script' ) );
	}

	
	  ////////////////////////////
	 /// PLUGIN SETTINGS PAGE ///
	////////////////////////////
	/**
	 * Add plugin settings page in WooCommerce plugin submenu.
	 */
	function add_plugin_options_page() {
		add_submenu_page(
			'woocommerce',									// $parent_slug
			self::PLUGIN_NAME,								// $page_title
			self::PLUGIN_NAME,								// $menu_title
			'manage_options',								// $capability
			self::SETTINGS_PAGE_SLUG,						// $menu_slug
			array( $this, 'render_plugin_settings_page' )	// callable $function
		);
	}

	/**
	 * Output the plugin settings page HTML.
	 * The settings page is a simple form, which is submitted to options.php.
	 * The form fields rendered via do_settings_sections() callback.
	 * The settings are saved in wp_options database table under a single serialized
	 * option with a name self::OPTION_NAME.
	 */
	function render_plugin_settings_page() {
		?>
		<h1>Simple Shipping Labels</h1>
		<p>Set the label parameters for generating simple shipping labels on the WooCommerce orders page.</p>
		<form action="options.php" method="post">
			<?php
				settings_fields( self::OPTION_GROUP_NAME );			// Output nonce, action, and option_page fields for a settings page.
				do_settings_sections( self::SETTINGS_PAGE_SLUG );	// Prints out all settings sections added to a particular settings page.
			?>
			<input type="submit" name="submit" class="button button-primary"  value="Save"/>
			<button type="button" id="button-reset-settings" class="button">Reset settings</button>
		</form>
		<script>
			var delete_settings_button = document.querySelector("#button-reset-settings");
			delete_settings_button.addEventListener('click', function() {
				
				// Make sure the user wants to delete saved plugin settings.
				if (!confirm("Delete saved plugin options and reset settings?")) {
					return;
				}

				// Show progress in button text.
				this.disabled = true;
				this.innerHTML = "Deleting saved plugin options...";
				
				// Send AJAX request to plugin API to reset plugin settings.
				var data = {
					'action': 'sslabels_api',
					'api_action': 'reset_options',
					'_ajax_nonce': '<?php echo wp_create_nonce('reset_options') ?>'
				};

				// Since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php.
				jQuery.post(ajaxurl, data, function(response) {
					location.reload();
				}).fail(function() {
					alert( "Could not delete saved plugin options. Please try again later or contact plugin support." );
					console.error(response);
					delete_settings_button.innerHTML = "Reset settings";
					delete_settings_button.disabled = false;
				});
			});
		</script>

		<?php
	}


	/**
	 * Register a single setting/option to store all plugin settings.
	 */
	function register_settings() {
		
		// Register the setting and data validation filter function.
		register_setting( self::OPTION_GROUP_NAME, self::OPTION_NAME, array( $this, 'plugin_options_validate' ) );
		
		// ##############################
		// ### Label settings section ###
		// ##############################
		$label_dimensions_section_slug = 'label_settings';
		add_settings_section( $label_dimensions_section_slug, 'Label settings', array( $this, 'settings_section_text' ), self::SETTINGS_PAGE_SLUG );
		add_settings_field( 'label-size', 'Label size', array( $this, 'setting_label_size'), self::SETTINGS_PAGE_SLUG, $label_dimensions_section_slug );
		add_settings_field( 'label-padding', 'Label padding', array( $this, 'setting_label_padding' ), self::SETTINGS_PAGE_SLUG, $label_dimensions_section_slug );
		add_settings_field( 'plugin_setting_test_labels', 'Test labels', array( $this, 'setting_test_labels'), self::SETTINGS_PAGE_SLUG, $label_dimensions_section_slug );

		// ################################################
		// ### Recipient/order address settings section ###
		// ################################################
		$recipient_section_slug = 'recipient_settings';
		add_settings_section( $recipient_section_slug, 'Recipient/order address section', array( $this, 'settings_section_text' ), self::SETTINGS_PAGE_SLUG );
		add_settings_field( 'plugin_setting_hide_base_country', 'Local shipping', array( $this, 'setting_hide_base_country'), self::SETTINGS_PAGE_SLUG, $recipient_section_slug );
		add_settings_field( 'plugin_setting_recipient_details_align', 'Fields text align', array( $this, 'setting_recipient_details_align'), self::SETTINGS_PAGE_SLUG, $recipient_section_slug );
	}


	/**
	 * Sanitize and validate plugin settings page form fields.
	 */
	function plugin_options_validate( $input ) {
		
		// Sanitize and validate user plugin settings.
		$sanitized_and_validated_input = array(
			'label_height'				=>	absint( $input['label_height'] ),
			'label_width'				=>	absint( $input['label_width'] ),
			'label_padding_vertical'	=>	absint( $input['label_padding_vertical'] ),
			'label_padding_horizontal'	=>	absint( $input['label_padding_horizontal'] ),
			'test_labels'				=>	wp_parse_id_list( $input['test_labels'] ),
			'plugin_version'			=> 	self::PLUGIN_VERSION,	// Add plugin version to the stored plugin settings option.
		);
		
		// If checkboxes are checked - they exist in $input array and we add them to $sanitized_input.
		if ( isset( $input['hide_base_country'] ) ) {
			$sanitized_and_validated_input['hide_base_country'] = 'on';
		}

		// Validate the recipient_align_fields option is one of the available options, otherwise use the default setting.
		if ( isset( $input['recipient_align_fields'] ) && in_array( $input['recipient_align_fields'], $this->recipient_align_fields_options, true) ) {		// `true` enables strict type checking
			$sanitized_and_validated_input['recipient_align_fields'] = $input['recipient_align_fields'];
		} else {
			$sanitized_and_validated_input['recipient_align_fields'] = $this->$default_settings['recipient_align_fields'];
		}
		
		// Return the sanitized and validated input (plugin settings) to save as option in wp_options database table.
		return $sanitized_and_validated_input;
	}


	  ////////////////////////////////////////////////
	 /// PLUGIN SETTINGS PAGE SECTIONS AND FIELDS ///
	////////////////////////////////////////////////
	function settings_section_text() {
		// echo "<p>Label dimensions section description.</p>";
	}
	
	// #####################
	// ### Label section ###
	// #####################
	function setting_label_size() {
		?>
		<p class="description">The horizontal and vertical sides lenghts of the label.</p>
		<ul>
			<li>
				<label for="label-height">Height :
				<input id="label-height" class="small-text" name="<?php echo self::OPTION_NAME; ?>[label_height]" type="number" min="10"  value="<?php echo esc_attr( $this->settings['label_height'] ); ?>" />
				mm</label>
			</li>
			<li>
				<label for="label-width">Width :
				<input id="label-width" class="small-text" name="<?php echo self::OPTION_NAME; ?>[label_width]" type="number" min="10"  value="<?php echo esc_attr( $this->settings['label_width'] ); ?>" />
				mm</label>
			</li>
		</ul>
		<p>Note: if you find empty labels (page overflows) in print dialogue - use 1mm less height.</p>
		<?php
	}
	
	function setting_label_padding() {
		?>
			<p class="description">The inner margins of the label.</p>
			<ul>
				<li>
					<label for="label-padding-top-bottom">Top and bottom :
					<input id="label-padding-top-bottom" class="small-text" name="<?php echo self::OPTION_NAME; ?>[label_padding_vertical]" type="number" min="0"  value="<?php echo esc_attr( $this->settings['label_padding_vertical'] ); ?>" />
					mm</label>
				</li>
				<li>
					<label for="label-padding-left-right">Left and right :
					<input id="label-padding-left-right" class="small-text" name="<?php echo self::OPTION_NAME; ?>[label_padding_horizontal]" type="number" min="0"  value="<?php echo esc_attr( $this->settings['label_padding_horizontal'] ); ?>" />
					mm</label>
				</li>
			</ul>
		<?php
	}

	
	function setting_test_labels() {
		?>
		<p class="description">Enter a comma separated list of order ids to test your saved settings.</p>
		<ul>
			<li>
				<input type="text" id="test-labels-list" class="regular-text" name="<?php echo self::OPTION_NAME; ?>[test_labels]" value="<?php echo implode(",", wp_parse_id_list( $this->settings['test_labels'] ) ) ?>" pattern="^[0-9]+(,[1-8]+)*$"/>
				<button id="button-generate-labels" type="button" class="button">Generate test labels</button>
				<p class="description">(save changes before generating test labels)</p>
			</li>
		</ul>

		<script type="text/javascript" >
			var generate_labels_button = document.querySelector("#button-generate-labels");
			generate_labels_button.addEventListener('click', function() {
				
				// Show progress in button text.
				this.innerHTML = "Generating...";
				this.disabled = true;

				// Send AJAX request to plugin API to generate labels page.
				var data = {
					'action': 'sslabels_api',
					'orders': document.querySelector("#test-labels-list").value.split(","),
					'api_action': 'generate_labels',
					'_ajax_nonce': '<?php echo wp_create_nonce('generate_labels'); ?>'
				};

				// Since 2.8 ajaxurl is always defined in the admin header and points to admin-ajax.php.
				jQuery.post(ajaxurl, data, function(response) {
					
					// Open new browser tab and set its content to the response labels page.
					var popup_window = window.open('','Shipping Labels');
					popup_window.document.write(response);		// Write the content of the document.
					popup_window.document.close();				// Finishes writing to a document.
					
					// Reset button to default text.
					generate_labels_button.innerHTML = "Generate test labels";
					generate_labels_button.disabled = false;

					// Try displaying the new window, but it might fail due to some pop-up blocker.
					try {
						popup_window.focus();
					} catch (e) {
						alert("Browser or extension pop-up blocker is enabled. Please add this site to your exceptions list for the generated labels page pop-up window.");
					}

				}).fail(function() {
					generate_labels_button.innerHTML = "Generate test labels";
					generate_labels_button.disabled = false;
					alert( "Could not generate labels." );
				});
				
			});
		</script>
		<?php
	}


	// #########################
	// ### Recipient section ###
	// #########################
	function setting_hide_base_country() {
		$checked_state = '';
		if ( isset($this->settings['hide_base_country']) ) {
			$checked_state = ' checked';
		}
		?>
		<label for="hide-base-country">
		<input id="hide-base-country" name="<?php echo self::OPTION_NAME; ?>[hide_base_country]" type="checkbox"<?php echo $checked_state; ?>/>Hide country field when shipping to store base country</label>
		<?php
	}

	
	function setting_recipient_details_align() {
		$selected_align_option = esc_html( $this->settings['recipient_align_fields'] );
		
		// Output the select field, its options and set the selected option.
		?>
		<select name="<?php echo self::OPTION_NAME; ?>[recipient_align_fields]">
			<?php
			foreach ($this->recipient_align_fields_options as $value) {
				if ($value == $selected_align_option) {
					echo '<option selected="selected" value="' . $value . '">' . $value. '</option>';
				} else {
					echo '<option value="' . $value . '">' . $value. '</option>';
				}
			}
			?>
		</select>
		<?php
	}


	  ///////////////////////////////
	 /// WooCommerce orders page ///
	///////////////////////////////
	/**
	 * Add custom column to WooCommerce orders table page.
	 */
	function add_print_buttons_column( $columns ) {
		$columns['shipping_label'] = __('Shipping labels');
		return $columns;
	}

	/**
	 * Populate the column with print buttons.
	 */
	function add_print_buttons( $column ) {
		global $post;
	 
		if ( 'shipping_label' === $column ) {
			$shipping_label_print_button_html = '<button type="button" class="button button-sslabels-print" ' 
			. 'value="' . $post->ID . '">'. __( 'Label' ) .'</button>';
			echo $shipping_label_print_button_html;
		}
	}


	/**
	 * Add JavaScript functionality to the "Print" buttons on the WooCommerce orders page.
	 * We enqueue the JavaScript script on that specific page (also known as the 'shop_order' screen).
	 * Then we pass it the plugin settings and additional style and script files paths to apply
	 * to the generated page of labels.
	 * We use wp_localize_script() to pass PHP variables to the enqueued JavaScript
	 */
	function add_orders_page_script( $hook ) {
		if ( ! is_user_logged_in() ) exit;
		
		if( 'edit.php' != $hook ) return;
		
		// Here we pass the stored options or default values to the orders page JavaScript,
		// for it to generate a shipping labels page with correct settings.
		$current_screen = get_current_screen();
		if ( 'shop_order' == $current_screen->post_type ) {
			
			// Add JavaScript functionality to the main orders page,
			// for the Shipping Labels column 'Print' buttons to use.
			wp_enqueue_script( 'sslabels-script', plugins_url( '/js/woocommerce-orders.js', __FILE__ ) );

			// Get the plugin settings options for JavaScript to generate customized labels, and pass it
			// to the enqueued JavaScript sctipt.
			$labels_page_localized_js_object = array(
				'_ajax_nonce'					=> wp_create_nonce('generate_labels'),
				'button_name'			=> __( 'Label' ),
				'button_active_name'	=> __( 'Generating' ) . '...'
			);
			wp_localize_script( 'sslabels-script', 'sslabels', $labels_page_localized_js_object );
		}
	}


	  ///////////////////////////////////
	 /// AJAX callbacks (plugin API) ///
	///////////////////////////////////
	/**
	 * Plugin AJAX endpoint that acts as the plugin API for the following actions:
	 * - Generate and return labels page for a list of order ids.
	 * - Reset plugin setting - remove store plugin settings option for wp_options database table and set the $settings field to $default_settings.
	 */
	function sslabels_api() {

		// Security checklist:
		// 1) API action exists.
		// 2) Verify nonce.
		// 3) Check user capabilities.
		// 4) Sanitize and Validate input.
		// 5) Escape output.
		$allowed_api_actions = [ 'generate_labels', 'reset_options' ];

		// If no 'api_action' or nonces exist in requesy - exit immediately.
		if ( !isset( $_REQUEST['api_action'] ) || !isset( $_REQUEST['_ajax_nonce'] ) ) {
			exit;
		}

		// Do the correct api action.
		switch($_REQUEST['api_action']) {
			
			case 'generate_labels':
				if (wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'generate_labels' ) && 
					current_user_can('edit_posts') && 
					isset( $_POST['orders'] )
					) {

					// Sanitize an array or comma/space-separated list of IDs (WooCommerce orders IDs in this case).
					$orders = wp_parse_id_list( $_POST['orders'] );		// Uses absint() to convert a value to non-negative integer.
					
					// Generate response HTML.
					$this->generate_shipping_labels_page_html( $orders );
				}
				break;
			
			case 'reset_options':
				if (wp_verify_nonce( $_REQUEST['_ajax_nonce'], 'reset_options' ) && current_user_can('manage_options') ) {
					delete_option( self::OPTION_NAME );
				}
				break;
			default:
				wp_die();
		}
		
		wp_die(); // This is required to terminate immediately and return a proper response.
	}


	  //////////////////////////////////////////////////////
	 /// Plugin activation, version control and updates ///
	//////////////////////////////////////////////////////
	/**
	 * Check if plugin files version equal to the version stored in database option.
	 * If default settings are still used - then there is no need to updated stored
	 * options and the plugin is considered up to date.
	 */
	function plugin_is_current_version() {
		
		// Use special php function to compare the stored option and the current
		// sslabels_PLUGIN_VERSION constant, defined in main simple-shipping-labels.php plugin file.
		return version_compare($this->settings['plugin_version'], self::PLUGIN_VERSION, '=') ? true : false;
	}

	/**
	 * Check plugin version and run update procedures if necessary.
	 */
	function check_plugin_version() {
		
		// No need to do anything if plugin version didn't change or no custom user options/settings were found.
		if ( $this->plugin_is_current_version() ) {
			return;
		}

		// In the future here will run plugin update procedures...

		// Update plugin version (we reach this code only if there were stored plugin options and the version was different).
		$options = get_option( self::OPTION_NAME );
		$options['plugin_version'] = self::PLUGIN_VERSION;
		update_option( 'self::OPTION_NAME', $options);
	}


	  /////////////////////////////////////////
	 /// Client labels page pop-up content ///
	/////////////////////////////////////////
	/**
	 * If a string contains Hebrew/Arabic characters - return a CSS class that is
	 * used to style rtl fields.
	 */
	function rtl_css_class($str) {
		$pattern = '/[\x{0591}-\x{07FF}\x{FB1D}-\x{FDFD}\x{FE70}-\x{FEFC}]/u';

		if ( preg_match($pattern, $str) )
		{
			echo " rtl";
		} else {
			echo "";
		}
	}


	/**
	 * Output HTML for shipping labels page.
	 * The HTML is used as the content of a client pop-up tab/window, 
	 * created by JavaScript in client browser, for the client to print.
	 */
	function generate_shipping_labels_page_html( $orders ) {
		?>
		<head>
		<title>Shipping Labels</title>
		<style>
		/* Page style */
		body {
			background-color: rgb(210, 214, 220);
			margin: 0px;
			padding: 0px;
		}

		/* Label style */
		.shipping-label {
			position: relative;
			box-sizing: border-box;
			display: flex;
			flex-direction: column;
			justify-content: center;
			align-items: center;
			width: <?php echo esc_html( $this->settings['label_width'] ) ?>mm;
			height: <?php echo esc_html( $this->settings['label_height'] ) ?>mm;
			padding: <?php echo esc_html( $this->settings['label_padding_vertical'] ) . "mm " . esc_html( $this->settings['label_padding_horizontal'] ) ."mm" ?>;
			margin: 20px;
			border-radius: 3mm;
			background-color: white;
			box-shadow: 0 1px 2px 0 rgba(60,64,67,.3), 0 1px 3px 1px rgba(60,64,67,.15);
			font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,Oxygen-Sans,Ubuntu,Cantarell,"Helvetica Neue",sans-serif;
		}

		/* Support for WordPress image alignment classes */
		.alignleft {
			float: left;
		}

		.alignright {
			float: right;
		}

		.aligncenter {
			display: block;
			margin-left: auto;
			margin-right: auto;
		}

		.recipient-container {
			flex: 1;
			width: 100%;
			display: flex;
			flex-direction: column;
			align-items: center;
			overflow: hidden;
		}

		.detail-container {
			flex: 1;
			box-sizing: border-box;
			display: flex;
			align-items: center;
			width: 100%;
			border: none;
			padding: 0px;
			margin: 0px;
			background-color: transparent;
			overflow: hidden;
		}

		.detail {
			background-color: transparent;
			box-sizing: border-box;
			overflow: hidden;
			resize: none;
		}

		[contenteditable]:focus {
			outline: none;
			background-color: rgb(195,207,221);
		}

		/* Customer details style */
		<?php
		// Customer fields alignment.
		$selected_align = $this->settings['recipient_align_fields'];
		if ($selected_align == "left") { 
			echo '.detail { text-align: left; } .detail-container { justify-content: flex-start; }';
		} else if ($selected_align == "right") { 
			echo '.detail { text-align: right; } .detail-container { justify-content: flex-end; }';
		} else {
			echo '.detail { text-align: center; } .detail-container { justify-content: center; }';
		}
		?>

		.rtl {
			direction: rtl;
		}

		.name {
			flex: 1.5;
			font-weight: bold;
		}

		.address {
			flex: 1.5;
		}

		.country {
			font-weight: bold;
		}

		.phone {
			align-items: baseline;
		}

		.order-id {
			vertical-align: text-top;
			height: fit-content;
			width: fit-content;
			position: absolute;
			right: 15px;
			bottom: 10px;
			font-size: 16px;
		}

		/*** Print style - hide redundant elements and styles ***/
		@media print {
			body {
				background-color: initial;
			}

			.shipping-label {
				page-break-after: always;
				border: none;
				border-radius: initial;
				box-shadow: none;
				margin: 0px;
			}
		}
		</style>
		</head>
		<body>
		<?php
		// Generate shipping label HTML for each order_id.
		foreach ($orders as $order_id)
		{
			// Try to get the WooCommerce order corresponding to the current order id (instance of WC_Order object).
			$order = wc_get_order( $order_id );
			
			// If current order id doesn't exist - continue to generating next order label.
			if (!$order) {
				continue;
			}

			// Get the customer shipping information details.
			$shipping_first_name = $order->get_shipping_first_name();
			$shipping_last_name  = $order->get_shipping_last_name();
			$shipping_company    = $order->get_shipping_company();
			$shipping_address_1  = $order->get_shipping_address_1();
			$shipping_address_2  = $order->get_shipping_address_2();
			$shipping_city       = $order->get_shipping_city();
			$shipping_state      = $order->get_shipping_state();
			$shipping_postcode   = $order->get_shipping_postcode();
			$shipping_country    = $order->get_shipping_country();
			$billing_phone       = $order->get_billing_phone();
			
			// For international shipping (shipping country != store base country)
			// get shipping country and state in English, by temporary switching WordPress environment locale.
			$is_shipping_to_base_country = WC()->countries->get_base_country() === $shipping_country ;			
			
			// Declare result containers.
			$country;
			$state;

			// If shipping to local address - use default WordPress $locale='default' language for local country and state shipping names.
			if ( $is_shipping_to_base_country ) {
				$country = WC()->countries->countries[$order->get_shipping_country()];
				$state  = ! empty( WC()->countries->get_states( $order->get_shipping_country() ) ) ? WC()->countries->get_states( $order->get_shipping_country() )[$order->get_shipping_state()] : '';

			// Else - switch to $locale='en_US' to generate international shipping labels in English. 
			} else {
				// Switch locale (WordPress language for translations) to get countries list in English, using WordPress locale_switcher.
				$locale_switched = switch_to_locale( $locale='en_US' );
				
				// Get full country/state names, using the WooCommerce countries.php and states.php files lists.
				$countries = include WC()->plugin_path() . '/i18n/countries.php';
				$country = $countries[ $order->get_shipping_country() ];

				$states = include WC()->plugin_path() . '/i18n/states.php';
				$state  = ! empty( $states[ $order->get_shipping_country() ] ) ? $states[ $order->get_shipping_country() ][$order->get_shipping_state()] : '';

				// When we are done, restore the initial WordPress locale.
				if ( $locale_switched ) {
					restore_previous_locale();
				}
			}

			// Generate current order shipping label element and append to the HTML result string.
			// To support Right-to-Left data, we asign ".rtl" class (for css direction/aligning styles) to all fields
			// containing RTL characters. To find those characters, we use Regex (regular expression) patterns in
			// $this->rtl_css_class() function.
			?>
			<div class="shipping-label">
				<div class="recipient-container recipient">
					<div class="detail-container name"><div contenteditable="true" class="detail<?php $this->rtl_css_class($shipping_first_name) ?>"><?php echo esc_html( $shipping_first_name ) . '&nbsp;' . $shipping_last_name ?></div></div>
					<div class="detail-container address"><div contenteditable="true" class="detail<?php $this->rtl_css_class($shipping_address_1) ?>"><?php echo esc_html( $shipping_address_1 ) . ( (strlen($shipping_address_2) == 0) ? "" : "&nbsp;" . esc_html( $shipping_address_2 ) ) ?></div></div>
					<div class="detail-container city"><div contenteditable="true" class="detail<?php $this->rtl_css_class($shipping_city) ?>"><?php echo esc_html( $shipping_city ) . "&nbsp;" . esc_html( $shipping_postcode ) . ( (strlen($shipping_state) == 0) ? "" : "&nbsp;" . esc_html( $shipping_state ) ) ?></div></div>
					<?php 
						if (!$is_shipping_to_base_country || ($is_shipping_to_base_country && !isset($this->settings['hide_base_country'])) ) {
							echo '<div class="detail-container country' . $this->rtl_css_class($state) . '"><div contenteditable="true" class="detail">' . esc_html( $country ) . '</div></div>';
						}
					?>
					<div class="detail-container phone"><div contenteditable="true" class="detail"> &#9742; <?php echo esc_html( $billing_phone ) ?></div></div>
				</div>
				<div class="order-id"><?php echo esc_html( $order_id ) ?></div>
			</div>
			<?php
		}
		?>
		<script>
			document.addEventListener('DOMContentLoaded', simpleShippingLabelsPageOnReady);
			
			/**
			 * This function adds custom 'input' event to trigger auto text fitting script
			 * when user edits any label field. It also triggers the attached event to intiaily 
			 * fit text in each detail field.
			 */
			function simpleShippingLabelsPageOnReady() {
				var details = document.querySelectorAll('.detail');
				var event = new Event('input', {
					bubbles: true,
					cancelable: true,
				});
				for (let i = 0; i < details.length; i++) {
					details[i].addEventListener('input', simpleShippingLabelsFitInput);
					details[i].dispatchEvent(event);
				}
			}

			/**
			 * This function tests elements content overflow/available space to grow and adjusts
			 * the font size accordingly.
			 */
			function simpleShippingLabelsFitInput() {
				while ( (this.innerText.length > 0) && (this.clientWidth < this.parentElement.clientWidth) && (this.clientHeight < this.parentElement.clientHeight) ) {
					this.style.fontSize = parseFloat(window.getComputedStyle(this).fontSize) + 0.3 + 'px';
				}
				while ( (this.clientWidth > this.parentElement.clientWidth) || (this.clientHeight > this.parentElement.clientHeight) ) {
					this.style.fontSize = parseFloat(window.getComputedStyle(this).fontSize) - 0.3 + 'px';
				}
			}
		</script>
		</body>
		<?php
	}


}
